"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.aggregateCSVReports = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const report_to_row_1 = require("./report-to-row");
const sync_1 = __importDefault(require("csv-stringify/lib/sync"));
const { readdir, writeFile } = fs_1.default.promises;
/** Combines the individual report CSV's from a folder into a single CSV file */
const aggregateCSVReports = async (dataDirPath) => {
    const reportsDirPath = path_1.default.join(dataDirPath, 'reports');
    const files = await readdir(reportsDirPath);
    const rows = [];
    let headers = null;
    for (const fileName of files) {
        if (fileName !== '.DS_Store') {
            const filePath = path_1.default.join(reportsDirPath, fileName);
            const fileContents = fs_1.default.readFileSync(filePath, 'utf8');
            // If headers aren't set yet, do it now
            if (!headers)
                headers = report_to_row_1.reportToRowHeaders(fileContents);
            const newRow = report_to_row_1.reportToRow(fileContents);
            if (newRow) {
                rows.push(newRow);
            }
            else {
                console.log(`Failed to bundle: ${fileName}`);
            }
        }
    }
    rows.unshift(headers);
    const aggregatedReportData = sync_1.default(rows);
    const writePath = path_1.default.join(dataDirPath, 'aggregatedMobileReport.csv');
    await writeFile(writePath, aggregatedReportData);
};
exports.aggregateCSVReports = aggregateCSVReports;
